<?php
/**
 * @author: Yusnel Rojas Garcia <yrojass@gmail.com>
 * @date: 9/22/12
 */
class FSerialGeneratorValidator extends CDefaultValueValidator
{

    public $unique = true;
    public $criteria = array();
    public $className;

    public $template = 'alfanum(4)-alfa(4)-num(4)';


   
    /**
     * Validates the attribute of the object.
     * @param CActiveRecord $object the object being validated
     * @param string $attribute the attribute being validated
     */
    protected function validateAttribute($object,$attribute)
    {
        if(!$this->setOnEmpty && !$this->unique)
            $object->$attribute=$this->generateSerial($this->template); //no need to be an active record
        else if(!$this->setOnEmpty)
            $object->$attribute=$this->getSerialForModel($object, $attribute);
        else
        {
            $value=$object->$attribute;
            if($value===null || $value===''){
                if (!$this->unique){
                    $object->$attribute=$this->generateSerial($this->template); //no need to be an active record
                }else{
                    $object->$attribute=$this->getSerialForModel($object, $attribute);
                }
            }
        }
    }

    public static function makeWord($n){
        $baseConsonants = 'BCDFGHJKLMNPQRSTVWXYZ';
        $baseVocals = 'AEIOU';
        $lengthConsonants = strlen($baseConsonants);
        $lengthVocals = strlen($baseVocals);
        $word = '';
        for ($i = 0; $i < $n; $i++){
            $targetBase = ($i%2==0)?$baseConsonants:$baseVocals;
            $targetLength = ($i%2==0)?$lengthConsonants:$lengthVocals;
            $word = $word . $targetBase[rand(0, $targetLength-1)];
        }
        return $word;
    }

    public static function makeNumber($n){
        $base = '0123456789';
        $length = strlen($base);
        $number = '';
        for ($i = 0; $i < $n; $i++){
            $number = $number . $base[rand(0, $length-1)];
        }
        return $number;
    }

    public static function makeWordNumber($n){
        $word = '';
        for ($i = 0; $i < $n; $i++){
            if (rand(0,1)>=0.5){
                $word .= self::makeNumber(1);
            }else{
                $word .= self::makeWord(1);
            }
        }
        return $word;
    }


    /**
     * @param CActiveRecord $object
     * @param $attribute
     * @return string
     */
    public function getSerialForModel($object, $attribute){
        $className=$this->className===null?get_class($object):Yii::import($this->className);
        $finder = CActiveRecord::model($className);
        do {
            $serial = self::generateSerial($this->template);
            $criteria = new CDbCriteria();
            $criteria->compare($attribute, $serial);
            $criteria->mergeWith($this->criteria);
            $criteria->limit = 1;

            $found = $finder->count($criteria) > 0;

        } while ($found && $this->unique);

        return $serial;
    }

    public static function generateSerial($template){
        return preg_replace_callback('/(?<type>(alfanum|alfa|num))\((?<length>\d+)\)+/i', array(self, 'templateReplace'), $template);
    }

    public static function templateReplace($match){
        switch($match['type']){
            case 'alfa':
                return self::makeWord($match['length']);
                break;
            case 'num':
                return self::makeNumber($match['length']);
                break;
            case 'alfanum':
                return self::makeWordNumber($match['length']);
                break;

            default:
                throw new CException('Unknown type in template ( wrong REGEXP ??? )');
        }
    }


}
